! ============================================================================
! Name        : main.f90
! Author      : Giancarlo Panichi
! Version     : 1.0
! Copyright   : LGPL
! Description : Fortran Statistic
! ============================================================================

program main
    ! Declare Variables
    implicit none
    character(len=1024), dimension(:), allocatable :: args
    integer :: argCount
    integer :: allocStatus, rdErr, i, iNum

    ! Check Argument
    argCount = command_argument_count()
    if (argCount == 0) then
            write (*,'(a)') "No command line arguments provided."
            stop
    end if

    ! ----------
    ! Allocate an array to hold the arguments.
    allocate(args(argCount), stat=allocStatus)
    if (allocStatus > 0) then
        write (*,'(a)') "Allocation error, program terminated."
        stop
    end if


    ! ----------
    ! Get each argument, one at a time, from system.
    do i = 1, argCount
        call get_command_argument(number=i,value=args(i))
    end do

    ! ----------
    ! Display arguments to screen.
    if (argCount == 0) then
         write (*,'(a)') "No command line arguments provided."
    else if (argCount == 1) then
            write (*,'(a)') "There was 1 command line argument."
         else
            write (*,'(a, i2, a)') "There were ", argCount, " command line arguments."
         end if
         write (*,'(/,a)') "The arguments were: "
         do i = 1, argCount
            write (*,'(a, a)') " ", trim(args(i))
         end do
         write (*,*)


    ! ----------
    ! Calculate
    call calculate(args(1),args(2))


    contains
    ! ------------
    ! Calculate operation using the file
    subroutine calculate(fileName, opName)
        character(len=*), intent(in) :: fileName
        character (len=3), intent(in) :: opName
        character(len=10) :: resultFile = "result.txt"

        integer :: i, rdopst, wropst, rdst, wrst
        character(len=20) :: line

        integer :: cvtErr
        integer :: num
        integer :: value

        write (*, '(a,1x,a)') "Filename: ", trim(fileName)
        write (*, '(a,1x,a)') "Operation: ", trim(opName)

        open(10, file=trim(fileName), status="old", action="read", position="rewind",iostat=rdopst )
        if (rdopst/=0) then
            write (*, '(a,1x,a)') "Error reading file: ", trim(fileName)
            stop
        endif
        write(*,*) "Input File Opened"

        open (unit=11, file=resultFile, status="replace", action="write", position="rewind", iostat=wropst)
        if (wropst > 0) then
            write(*,*) "Cannot open the result.txt file."
            close(10)
            stop
        end if
        write(*,*) "File result.txt Opened"


        ! read first line from input file
        read (10, '(a)', iostat=rdst) line
        ! if end of file
        if (rdst >0) then
            write(11,*) "Error reading input file."
            close(10)
            close(11)
            stop
        endif
        if (rdst <0) then
            write (11,*) "Attention, The input file is empty"
            close(10)
            close(11)
            stop
        endif

        read (line, '(i10)', iostat=cvtErr) num
        if (cvtErr /= 0 ) then
             write (11,*) "Error, the input file contains a invalid integer value: "
             write (11,*) line
             close(10)
             close(11)
             stop
        endif
        value = num

        ! read lines from input file
        i = 1
        do
            read (10, '(a)', iostat=rdst) line
            ! if end of file
            if (rdst >0) then
                write(11,*) "Error reading input file."
                close(10)
                close(11)
                stop
            endif
            if (rdst <0) then
                exit
            endif
            read (line, '(i10)', iostat=cvtErr) num
            if (cvtErr /= 0 ) then
                write (11,*) "Error, the input file contains a invalid integer value: "
                write (11,*) line
                close(10)
                close(11)
                stop
            endif
            select case (opName)
                case ("avg")
                    value = value+num
                case ("sum")
                    value = value+num
                case ("max")
                    if(num>value) then
                        value = num
                    end if
                case ("min")
                    if(num<value) then
                        value = num
                    end if
                case default
                    value = 0
            end select
            i = i + 1
        end do

        if(opName=="avg") then
          value=value/i
        endif

        write(*,*) "Write result in result.txt"
        write(11,*) "The result is: ", value
        close(10)
        close(11)
        return
    end subroutine calculate

end program main
